/*****************************************************************************/
#include "tpcclibConfig.h"
/*****************************************************************************/
#include "tpcextensions.h"
#include "test_tpccm.h"
/*****************************************************************************/

/*****************************************************************************/
int test_simDispersion(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  int ret;

  if(verbose>1) printf("\n testing with empty data \n");
  ret=simDispersion(NULL, NULL, 0, 0.0, 0.0, NULL);
  if(verbose>2) printf("did not segfault, that is good.\n");
  if(ret==0) return(1);
  if(verbose>2) printf("\n more testing with empty data \n");
  ret=simDispersion(NULL, NULL, 1000, 2.0, 5.0, NULL);
  if(verbose>2) printf("did not segfault, that is good.\n");
  if(ret==0) return(2);

  TAC input, output;
  tacInit(&input); tacInit(&output);

  if(verbose>2) printf("\n yet more testing with empty data \n");
  ret=simDispersion(input.x, input.x, 1000, 2.0, 5.0, NULL);
  if(verbose>2) printf("did not segfault, that is good.\n");
  if(ret==0) return(3);

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>1) printf("\n testing with perfect bolus (nonrealistic for radiowater)\n");
  {
    PAR par; parInit(&par); parExamplePerfectBolus(&par, NULL);
    ret=tacAllocateWithPAR(&input, &par, 6001, status);
    if(ret!=TPCERROR_OK) {parFree(&par); tacFree(&input); return(100);}
    /* set sample times */
    double fdur=1.0;
    for(int i=0; i<input.sampleNr; i++) {
      input.x1[i]=(double)i*fdur;
      input.x2[i]=input.x1[i]+fdur; input.x[i]=0.5*(input.x1[i]+input.x2[i]);
    }
    /* compute input values */
    ret=mfEvalFrameY(modelCode(par.r[0].model), modelParNr(par.r[0].model),
                     par.r[0].p, input.sampleNr, input.x1, input.x2, input.c[0].y, 
                     verbose-100);
    if(ret) {tacFree(&input); parFree(&par); return(101);}
    parFree(&par);
  }

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>1) printf("\n testing with dispersion constants set to zero \n");
  {
    double y2[input.sampleNr], buf[input.sampleNr];
    for(int i=0; i<input.sampleNr; i++) y2[i]=input.c[0].y[i];
    ret=simDispersion(input.x, y2, input.sampleNr, 0.0, 0.0, buf);
    if(ret) {tacFree(&input); return(1001);}
    ret=doubleArrayMatch(y2, input.c[0].y, input.sampleNr, 1.0E-100);
    if(!ret) {tacFree(&input); return(1002);}

    for(int i=0; i<input.sampleNr; i++) y2[i]=input.c[0].y[i];
    ret=simDispersion(input.x, y2, input.sampleNr, 0.0, 0.0, NULL);
    if(ret) {tacFree(&input); return(1011);}
    ret=doubleArrayMatch(y2, input.c[0].y, input.sampleNr, 1.0E-100);
    if(!ret) {tacFree(&input); return(1012);}
  }

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>1) printf("\n testing with the first dispersion constant \n");
  {
    double y2[input.sampleNr], buf[input.sampleNr];
    for(int i=0; i<input.sampleNr; i++) y2[i]=input.c[0].y[i];
    ret=simDispersion(input.x, y2, input.sampleNr, 5.0, 0.0, buf);
    if(ret) {tacFree(&input); return(1101);}
    ret=doubleArrayMatch(y2, input.c[0].y, input.sampleNr, 0.1);
    if(ret) {tacFree(&input); return(1102);}
    ret=simC1(input.x, input.c[0].y, input.sampleNr, 1.0/5.0, 1.0/5.0, buf);
    if(ret) {tacFree(&input); return(1103);}
    ret=doubleArrayMatch(y2, buf, input.sampleNr, 0.1);
    if(!ret) {tacFree(&input); return(1104);}

    for(int i=0; i<input.sampleNr; i++) y2[i]=input.c[0].y[i];
    ret=simDispersion(input.x, y2, input.sampleNr, 5.0, 0.0, NULL);
    if(ret) {tacFree(&input); return(1111);}
    ret=doubleArrayMatch(y2, input.c[0].y, input.sampleNr, 0.1);
    if(ret) {tacFree(&input); return(1112);}
    ret=doubleArrayMatch(y2, buf, input.sampleNr, 0.1);
    if(!ret) {tacFree(&input); return(1114);}
  }

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>1) printf("\n testing with the second dispersion constant \n");
  {
    double y2[input.sampleNr], buf[input.sampleNr];
    for(int i=0; i<input.sampleNr; i++) y2[i]=input.c[0].y[i];
    ret=simDispersion(input.x, y2, input.sampleNr, 0.0, 5.0, buf);
    if(ret) {tacFree(&input); return(1201);}
    ret=doubleArrayMatch(y2, input.c[0].y, input.sampleNr, 0.1);
    if(ret) {tacFree(&input); return(1202);}
    ret=simC1(input.x, input.c[0].y, input.sampleNr, 1.0/5.0, 1.0/5.0, buf);
    if(ret) {tacFree(&input); return(1203);}
    ret=doubleArrayMatch(y2, buf, input.sampleNr, 0.1);
    if(!ret) {tacFree(&input); return(1204);}

    for(int i=0; i<input.sampleNr; i++) y2[i]=input.c[0].y[i];
    ret=simDispersion(input.x, y2, input.sampleNr, 0.0, 5.0, NULL);
    if(ret) {tacFree(&input); return(1211);}
    ret=doubleArrayMatch(y2, input.c[0].y, input.sampleNr, 0.1);
    if(ret) {tacFree(&input); return(1212);}
    ret=doubleArrayMatch(y2, buf, input.sampleNr, 0.1);
    if(!ret) {tacFree(&input); return(1214);}
  }

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>1) printf("\n testing with both dispersion constants \n");
  {
    double y2[input.sampleNr], buf[input.sampleNr], buf2[input.sampleNr];
    for(int i=0; i<input.sampleNr; i++) y2[i]=input.c[0].y[i];
    ret=simDispersion(input.x, y2, input.sampleNr, 2.0, 3.0, buf);
    if(ret) {tacFree(&input); return(1301);}
    ret=doubleArrayMatch(y2, input.c[0].y, input.sampleNr, 0.1);
    if(ret) {tacFree(&input); return(1302);}
    ret=simC1(input.x, input.c[0].y, input.sampleNr, 1.0/2.0, 1.0/2.0, buf);
    if(ret) {tacFree(&input); return(1303);}
    ret=simC1(input.x, buf, input.sampleNr, 1.0/3.0, 1.0/3.0, buf2);
    if(ret) {tacFree(&input); return(1304);}
    ret=doubleArrayMatch(y2, buf2, input.sampleNr, 0.1);
    if(!ret) {tacFree(&input); return(1305);}

    for(int i=0; i<input.sampleNr; i++) y2[i]=input.c[0].y[i];
    ret=simDispersion(input.x, y2, input.sampleNr, 2.0, 3.0, NULL);
    if(ret) {tacFree(&input); return(1311);}
    ret=doubleArrayMatch(y2, input.c[0].y, input.sampleNr, 0.1);
    if(ret) {tacFree(&input); return(1312);}
    ret=doubleArrayMatch(y2, buf2, input.sampleNr, 0.1);
    if(!ret) {tacFree(&input); return(1315);}
  }

  tacFree(&input);

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_corDispersion(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  int ret;

  if(verbose>1) printf("\n testing with empty data \n");
  ret=corDispersion(NULL, NULL, 0, 0.0, NULL);
  if(verbose>2) printf("did not segfault, that is good.\n");
  if(ret==0) return(1);
  if(verbose>2) printf("\n more testing with empty data \n");
  ret=corDispersion(NULL, NULL, 1000, 2.0, NULL);
  if(verbose>2) printf("did not segfault, that is good.\n");
  if(ret==0) return(2);

  TAC input, output;
  tacInit(&input); tacInit(&output);

  if(verbose>2) printf("\n yet more testing with empty data \n");
  ret=corDispersion(input.x, input.x, 1000, 2.0, NULL);
  if(verbose>2) printf("did not segfault, that is good.\n");
  if(ret==0) return(3);


  statusSet(status, __func__, __FILE__, __LINE__, 0);
  {
    if(verbose>1) printf("\n make dispersion-free perfect bolus (nonrealistic for radiowater)\n");
    PAR par; parInit(&par); parExamplePerfectBolus(&par, NULL);
    ret=tacAllocateWithPAR(&input, &par, 1001, status);
    if(ret!=TPCERROR_OK) {parFree(&par); tacFree(&input); return(100);}
    /* set sample times */
    double fdur=0.02;
    for(int i=0; i<input.sampleNr; i++) {
      input.x1[i]=(double)i*fdur;
      input.x2[i]=input.x1[i]+fdur; input.x[i]=0.5*(input.x1[i]+input.x2[i]);
    }
    /* compute input values */
    ret=mfEvalFrameY(modelCode(par.r[0].model), modelParNr(par.r[0].model),
                     par.r[0].p, input.sampleNr, input.x1, input.x2, input.c[0].y, 
                     verbose-100);
    if(ret) {tacFree(&input); parFree(&par); return(101);}
    parFree(&par);
    if(verbose>1) printf("\n add dispersion\n");
    if(tacDuplicate(&input, &output) || 
       simDispersion(output.x, output.c[0].y, output.sampleNr, 3.0, 0.0, NULL))
    {
      tacFree(&input); tacFree(&output); return(102);
    }
  }

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  {
    if(verbose>1) printf("\n try to correct for the added dispersion\n");
    double cy[output.sampleNr];
    for(int i=0; i<output.sampleNr; i++) cy[i]=output.c[0].y[i];
    ret=corDispersion(output.x, cy, output.sampleNr, 3.0, NULL);
    if(ret) {tacFree(&input); tacFree(&output); return(1101);}
    if(verbose>3) {
      for(int i=0; i<input.sampleNr; i++) printf("%9.3f\t\t%12.3e\t%12.3e\t%12.3e\t\t%g\n",
          input.x[i], input.c[0].y[i], output.c[0].y[i], cy[i], cy[i]-input.c[0].y[i]);
    }
    ret=doubleArrayMatch(input.c[0].y, cy, input.sampleNr, 0.05);
    if(!ret) {tacFree(&input); tacFree(&output); return(1102);}
  }

  tacFree(&input); tacFree(&output);

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
