/** @file tpclinopt.h
 *  @brief Header file for libtpclinopt.
 *  @details Header file for template library libtpclinopt.
 *  @author Vesa Oikonen
 */
#ifndef _TPCLINOPT_H_
#define _TPCLINOPT_H_
/*****************************************************************************/

/*****************************************************************************/
#include "tpcclibConfig.h"
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
/*****************************************************************************/
#include "tpcextensions.h"
/*****************************************************************************/

/*****************************************************************************/
/* regression */
int fitLine(
  double *x, double *y, const int n, double *m, double *c
);
int fitLinePearson(
  double *x, double *y, const int n,
  double *m, double *msd, double *c, double *csd, double *d, double *dsd, 
  double *r, double *ysd
);
int highestSlope(
  double *x, double *y, const int n, const int slope_n, double x_start,
  double *m, double *yi, double *xi, double *xh
);
/*****************************************************************************/

/*****************************************************************************/
/* qrlsq */
int qrLSQ(
  double **mat, double *rhs, double *sol, 
  const unsigned int rows, const unsigned int cols, double *r2
);
int qr(
  double **A, int m, int n, double *B, double *X, double *rnorm,
  double *tau, double *res, double **wws, double *ws
);
int qr_decomp(
  double **a, int M, int N, double *tau, double **cchain, double *chain
);
int qr_solve(
  double **QR, int M, int N, double *tau, double *b, double *x,
  double *residual, double *resNorm, double **cchain, double *chain
);
int qrWeight(
  int N, int M, double **A, double *b, double *weight, double *ws
);
int qrWeightRm(
  int N, int M, double **A, double *b, double *weight, double *ws
);
int qrSimpleLSQ(double **A, double *B, int M, int N, double *X, double *r2);
int qrLH(
  const unsigned int m, const unsigned int n,
  double *a, double *b, double *x, double *r2
);
/*****************************************************************************/

/*****************************************************************************/
/* hholder */
double householder_transform(
  double *vector, int size
);
int householder_hm(
  double tau, double *vector, double **matrix, int rowNr, int columnNr
);
int householder_hv(
  double tau, int size, double *v, double *w
);
double householder_norm(
  double *v, int size
);
/*****************************************************************************/

/*****************************************************************************/
/* nnls */
int nnls(
  double **a, int m, int n, double *b, double *x,
  double *rnorm, double *w, double *zz, int *index
);
int nnlsWght(
  int N, int M, double **A, double *b, double *weight
);
int nnlsWghtSquared(
  int N, int M, double **A, double *b, double *sweight
);
/*****************************************************************************/

/*****************************************************************************/
/* bvls */
int bvls(
  int key, const /*unsigned*/ int m, const /*unsigned*/ int n,
  double *a, double *b, double *bl, double *bu, double *x,
  double *w, double *act, double *zz, int *istate, int *iter,
  int verbose
);
int llsqWght(int N, int M, double **A, double *a, double *b, double *weight);
int llsqWghtSquared(int N, int M, double **A, double *a, double *b, double *weight);
/*****************************************************************************/

/*****************************************************************************/
#endif /* _TPCLINOPT_H_ */
